/*!
**    Notice:  Gubby Copyright 2003 Christian Stigen Larsen
**    Author:  <csl@sublevel3.org>
**
**    $Id: gubbyopts.cpp,v 1.11 2004/12/15 11:41:15 cslsublevel3org Exp $
**
**    This file is part of Gubby.
**
**    Gubby is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    Gubby is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Gubby; if not, write to the Free Software
**    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

#include "gubbyopts.h"

#include <string>
#include <iostream>
#include "guess.h"
#include "exists.h"
#include "die.h"

struct goption gubbyopts::options[] = {
	{"help", 0, 0, 'h'},
	{"version", 0, 0, 'v'},
	{"clear", 0, 0, 'c'},
	{"mailexec", 1, 0, 'm'},
	{"maildir", 1, 0, 'M'},
	{"mailprefix", 1, 0, 'p'},
	{"mailcompose", 1, 0, 'C'},
	{"once", 0, 0, 'o'},
	{"loop", 0, 0, 'l'},
	{"verbose", 0, 0, 'V'},
	{"wide", 0, 0, 'w'},
	{"beep", 0, 0, 'b'},
	{"nobeepdevnull", 0, 0, 's'},
	{"hidedevnull", 0, 0, 'H'},
	{0, 0, 0, 0}
};

const char* gubbyopts::helpopts[] = {
	"Print help and exit",
	"Print version info and exit",
	"Clear contents of Procmail logfile",
	"Specify program to read mailfolders with",
	"Specify directory where mailfolders are stored",
	"Specify mailprogram arguments to choose a mailfolder",
	"Specify mailprogram arguments to compose a new mail",
	"Print new mail and exit",
	"Start in curses screen",
	"Be verbose: Send warnings and notices to stderr",
	"When printing new mail and exit, include folder contents",
	"Beep on new mail (terminal bell)",
	"Do not beep when new mail folder is /dev/null",
	"Do not show /dev/null in the folderview",
	0
};


gubbyopts::gubbyopts(int argc, char* argv[])
	: progname(argv[0]),
	optmask("hvlcowbVm:M:p:C:sH")
{
	getopts::parse(argc, argv, optmask);
}

void gubbyopts::checkopts() {

	/*
	if ( verbose == false )
		if ( freopen("/dev/null", "w", stderr) == 0 )
			die("Could not redirect stderr to /dev/null");
	*/

	// No specified logfile? Try to guess it
	if ( logfile.size() == 0 ) {
		guess glogfile;
		logfile = glogfile.str();
	}

	if ( !exists(logfile) ) {
		std::string serr("Could not find Procmail logfile '");
		serr += logfile;
		serr += "'";
		die(serr);
	}
}

void gubbyopts::gotopt(const char ch, const char* value) {
	switch ( ch ) {
	case 'h':
		printversion();
		printhelp_();
		printurl();
		exit(0);
		break;

	case 'v':
		printversion();
		exit(0);
		break;

	case 'm':
		mailexec = value;
		break;

	case 'l':
		loop = true;
		break;

	case 'o':
		loop = false;
		break;

	case 'p':
		mailprefix = value;
		break;

	case 'M':
		maildir = value;
		break;

	case 'c':
		clearlog = true;
		break;

	case 'V':
		verbose = true;
		break;

	case 'C':
		mailcompose = value;
		break;

	case 'w':
		widelisting = true;
		break;

	case 'b':
		beep = true;
		break;

	case 's':
		nobeepdevnull = true;
		break;

	case 'H':
		hidedevnull = true;
		break;

	case '?':
	default:
		std::cout << std::endl;
		printversion();
		printhelp_();
		printauthor();
		exit(1);
		break;
	}
}

void gubbyopts::printhelp_() const {
	std::cout <<	"\n"
			"Usage: " << progname << " [option(s)] [logfile]\n"
			"  Logfile is the Procmail logfile to monitor.\n"
			"\n"
			"Options\n";

	for ( int n(0); options[n].name!=0; ++n ) {
		std::cout <<	"  -" << static_cast<char>(options[n].val) <<
				"    " << helpopts[n] << '\n';
	}
	
	std::cout << std::endl;
}
