/*!
**    Notice:  Gubby Copyright 2003 Christian Stigen Larsen
**    Author:  <csl@sublevel3.org>
**
**    $Id: keys.cpp,v 1.11 2004/12/15 11:41:15 cslsublevel3org Exp $
**
**    This file is part of Gubby.
**
**    Gubby is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    Gubby is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Gubby; if not, write to the Free Software
**    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

#include "keys.h"

#ifndef USE_STRSTREAM
#include <sstream>
#else
#include <strstream>
#endif

#include <iomanip>

//! Keys without name are "invisible" (not shown in help screens)
static struct key keys[] = {
	{'q', "Quit", "Quit Gubby", GKEY_QUIT},
	{'?', "Help", "Show help", GKEY_HELP},
	{KEY_F(1), 0, 0, GKEY_HELP},
	{'r', "Clear", "Empty contents of Procmail logfile", GKEY_CLEARLOG},
	{'g', 0, "Launch mail client on selected mailbox", GKEY_OPEN},
	{13, 0, 0, GKEY_OPEN},
	{KEY_ENTER, 0, 0, GKEY_OPEN},
	{'o', 0, "Launch mail client", GKEY_MAILPROG},
	{'c', 0, "Launch mail client for writing new message", GKEY_COMPOSE},
	{'l', 0, "Redraw the display", GKEY_REDRAW},
	{'k', 0, "Move up", GKEY_MOVEUP},
	{KEY_UP, 0, 0, GKEY_MOVEUP},
	{'j', 0, "Move down", GKEY_MOVEDOWN},
	{KEY_DOWN, 0, 0, GKEY_MOVEDOWN},
	{KEY_RIGHT, 0, 0, GKEY_MOVEEND},
	{KEY_LEFT, 0, 0, GKEY_MOVETOP},
	{'=', 0, 0, GKEY_MOVETOP},
	{'*', 0, 0, GKEY_MOVEEND},
	{KEY_HOME, 0, 0, GKEY_MOVETOP},
	{KEY_END, 0, 0, GKEY_MOVEEND},
	{'h', "Expand/Contract", "Expand folder email contents", GKEY_EXPAND},
	{'b', "Toggle beep", "Toggles beeping on new mail", GKEY_TOGGLE_BEEP},
	{'m', "Mark as read", "Marks folder as read", GKEY_MARKASREAD},
	{'d', "Remove folder", "Marks folder as read, and removes from logfile as well (experimental)", GKEY_REMOVEFOLDER},
	{0, 0, 0, GKEY_NONE}
};

keycmd keytoenum(const chtype ch) {
	for ( int p(0); keys[p].ch!=0; ++p ) {
		if ( keys[p].ch == ch )
			return keys[p].cmd;
	}

	return GKEY_NONE;
}

char findkey(const keycmd key) {
	for ( int p(0); keys[p].ch!=0; ++p ) {
		if ( keys[p].cmd == key )
			return keys[p].ch;
	}

	return '?';
}

void printkeys(WINDOW* w) {
	for ( int p(0); keys[p].cmd!=GKEY_NONE; ++p ) {
		if ( keys[p].help!=0 )
			wprintw(w, "     %c     %s\n", keys[p].ch, keys[p].help);
	}
}

const std::string keysinfo() {
#ifndef USE_STRSTREAM
	std::stringstream ss;
#else
	std::strstream ss;
#endif

	// fill keyinfo
	for ( int p(0); keys[p].ch != 0; ++p )
		if ( keys[p].name != 0 ) {

			ss << static_cast<char>(keys[p].ch) <<
				":" <<
				keys[p].name <<
				" ";
		}

	return ss.str();
}
