/*!
**    Notice:  Gubby Copyright 2003 Christian Stigen Larsen
**    Author:  <csl@sublevel3.org>
**
**    $Id: screens.cpp,v 1.17 2004/12/15 11:41:15 cslsublevel3org Exp $
**
**    This file is part of Gubby.
**
**    Gubby is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    Gubby is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Gubby; if not, write to the Free Software
**    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

#include "screens.h"
#include "filechange.h"

#ifndef USE_STRSTREAM
#include <sstream>
#else
#include <strstream> // for GCC 2.95.2 (archane, deprecated sstream)
#endif

class screen : public curswin {
public:
	screen() {
	}

	screen(const int y, const int x) : curswin(y, x) {
	}

	screen(WINDOW* p) : curswin(p) {
	}

	void wprintintense(const bool colors, const int y, const int x, const std::string& s);
	void wprintversion(const gubbycfg& gb);
	void wprintpointer(const bool colors, const int y);
};

void screen::wprintintense(const bool colors, const int y, const int x, const std::string& s) {
	if ( colors )
		wattrset(ptr(), COLOR_PAIR(2));
	else
		wattron(ptr(), A_REVERSE);

	wattron(ptr(), A_BOLD);
	mvwhline(ptr(), y, 0, ' ', width());
	mvwaddstr(ptr(), y, x, s.c_str());
}

void screen::wprintversion(const gubbycfg& gb) {
	wattrset(ptr(), COLOR_PAIR(1));

	std::string s("v");
	s += gb.version;
	s += " (";
	s += gb.year;
	s += "-";
	s += gb.date;
	s += ")";

	mvwaddstr(ptr(), height() - 1, 0, s.c_str());
}

void screen::wprintpointer(const bool colors, const int y) {
	if ( colors )
		wattrset(ptr(), COLOR_PAIR(3));
	else
		wattrset(ptr(), A_REVERSE);
		
	wattron(ptr(), A_BOLD);
	mvwaddstr(ptr(), y+1, 0, "->");
	wmove(ptr(), 1+y, width() - 1);
	wattroff(ptr(), A_BOLD);
}

screens::screens(gubbycfg& gb_, Folders& folders_)
	: gb(gb_),
	folders(folders_),
	curspos(0)
{
}

void screens::run() {

	if ( gb.clearlog ) {
		gb.clearlogfile();
		exit(0);
	}

	if ( gb.loop ) {
		startcurses(curses);
		mainscreen(curses);
	} else
		printfolders(gb.widelisting);
}


void screens::showhelp(const bool colors) {
	screen help;

	bool printscreen(true);
	std::string title("Help for index");
	std::string keybar;
	keybar += static_cast<char>(findkey(GKEY_QUIT));
	keybar += ":";
	keybar += "Exit";

	do {
		if ( help.resized() ) {
			werase(help.ptr());
			printscreen = true;
		}

		if ( printscreen ) {
			printscreen = false;

			help.wprintintense(colors, 0, 0, title);

			wattrset(help.ptr(), COLOR_PAIR(1));
			wattron(help.ptr(), A_BOLD | A_UNDERLINE);
			mvwaddstr(help.ptr(), 2, 2, "Main menu keys:");
			wattroff(help.ptr(), A_BOLD | A_UNDERLINE);

			wmove(help.ptr(), 4, 0);
			printkeys(help.ptr());

			wprintw(help.ptr(), "\n\n  ");
			wattron(help.ptr(), A_BOLD | A_UNDERLINE);
			wprintw(help.ptr(), "Homepage:\n");
			wattroff(help.ptr(), A_BOLD | A_UNDERLINE);
			wprintw(help.ptr(), "\n    %s", gb.url.c_str());

			//wattrset(help.ptr(), COLOR_PAIR(2));
			//wattron(help.ptr(), A_BOLD);
			//mvwhline(help.ptr(), help.height() - 2, 0, ' ', help.width());
			//mvwaddstr(help.ptr(), help.height() - 2, 0, keybar.c_str());
			help.wprintintense(colors, help.height() - 2, 0, keybar.c_str());


			help.wprintversion(gb);

			std::string wrt("Written by ");
			wrt += gb.authorshort;
			wrt += " <";
			wrt += gb.email;
			wrt += ">";
			wattrset(help.ptr(), COLOR_PAIR(1));
			mvwaddstr(help.ptr(), help.height() - 1, help.width() - wrt.size(), wrt.c_str());
			wattrset(help.ptr(), COLOR_PAIR(2));
			wmove(help.ptr(), 0, help.width() -1);
			wattroff(help.ptr(), A_BOLD);
		}

		wrefresh(help.ptr());

	} while ( keytoenum(wgetch(help.ptr())) != GKEY_QUIT );
}

bool screens::exitkey(const keycmd ch,
	bool& redrawall,
	initcurs& win,
	bool& update,
	bool& shelledout) {
	
	std::string cmd;
	update = true;

	switch ( ch ) {
		case GKEY_NONE:
			update = false;
			break;

		case GKEY_CLEARLOG:
			gb.clearlogfile();
			redrawall = true;
			break;

		case GKEY_COMPOSE:
			cmd = gb.mailexec;
			cmd += " ";
			cmd += gb.mailcompose;

			win.shell(cmd);
			redrawall = shelledout = true;
			break;

		case GKEY_REDRAW:
			redrawall = true;
			break;

		case GKEY_QUIT:
			return true;

		case GKEY_MOVEDOWN:
			if ( gb.expanded != -1 )
				redrawall = true;

			if ( ++curspos >= folders.size() )
				curspos = 0;
			break;

		case GKEY_MOVEUP:
			if ( gb.expanded != -1 )
				redrawall = true;

			if ( curspos == 0 ) {
				if ( folders.size() > 0 )
					curspos = folders.size()-1;
			} else
				--curspos;
			break;

		case GKEY_MOVETOP:
			if ( gb.expanded != -1 )
				redrawall = true;

			curspos = 0;
			break;

		case GKEY_MOVEEND:
			if ( gb.expanded != -1 )
				redrawall = true;

			if ( folders.size() > 0 )
				curspos = folders.size()-1;
			else
				curspos = 0;
			break;

		case GKEY_MAILPROG:
			win.shell(gb.mailexec);
			redrawall = shelledout = true;
			break;

		case GKEY_HELP:
			showhelp(win.hascolors());
			redrawall = true;
			break;

		case GKEY_TOGGLE_BEEP:
			gb.beep = !gb.beep;
			redrawall = true;
			break;

		case GKEY_EXPAND:
			if ( gb.expanded == -1 )
				gb.expanded = curspos;
			else
				gb.expanded = -1;
			redrawall = true;
			break;

		case GKEY_MARKASREAD:
			if ( !folders.empty() ) {
				folders.remove(folders[curspos].name());
				if ( folders.size() <= curspos )
					curspos = folders.size()-1;
				redrawall = true;
			}
			break;

		case GKEY_REMOVEFOLDER:
			if ( !folders.empty() ) {
				folders.remove(folders[curspos].name());
				if ( folders.size() <= curspos )
					curspos = folders.size()-1;
				folders.write(gb.logfile); // write changes (experimental)
				redrawall = true;
			}
			break;

		case GKEY_OPEN:
			if ( folders.size() != 0 ) {
				if ( folders[curspos].vmail.size() != 0 ) {
					std::string cmd(gb.mailexec);
					cmd += " ";
					cmd += gb.mailprefix;
					cmd += " ";

					if ( folders[curspos].name()[0] != '/' )
						cmd += gb.maildir;

					cmd += "/";
					cmd += folders[curspos].name();

					std::string dir( gb.maildir );
					std::string::size_type pos = dir.find("$HOME");

			                if ( pos != std::string::npos )
						dir.replace(pos, 5, getenv("HOME"));

					win.shell(cmd);
					redrawall = shelledout = true;
				}
			}
			break;

		default:
			update = false;
			break;
	}

	return false;
}

void screens::wprintfolders(WINDOW* w, const int expanded, const int width, const bool colors) {

	unsigned int pos(0), offsy(1);

	for ( Folders::const_iterator folder = folders.begin();
		folder != folders.end(); ++folder )
	{
		pos = ( folder - folders.begin() );

		wmove(w, offsy+pos, 0);
		wclrtoeol(w);

		if ( pos != curspos )
			wattrset(w, COLOR_PAIR(1));
		else {
			wattrset(w, COLOR_PAIR(3));
			if ( !colors) wattron(w, A_REVERSE);
			mvwhline(w, offsy+pos, 0, ' ', width);
		}

		wattron(w, A_BOLD);
		mvwprintw(w, offsy+pos, 1, "  %5d   %s",
			(*folder).vmail.size(),
			(*folder).name().c_str());

		// Mark folder with newest mail
		if ( (*folder).name() == folders.newest() )
			mvwprintw(w, offsy+pos, 9, "N");

		if ( expanded != -1 && pos == curspos ) {
			wattron(w, A_NORMAL);
			wattrset(w, COLOR_PAIR(1));

			int xpos(6);
			
			for ( std::vector<mail>::const_iterator p = (*folder).vmail.begin();
				p!=(*folder).vmail.end(); ++p )
			{
				std::string smail( "     " );
				smail += (*p).from;
				smail += ": ";
				smail += (*p).subj;

				// chop
				if ( smail.size() > static_cast<unsigned int>(width-xpos) )
					smail.erase(width-xpos, smail.size()-width+xpos);

				mvwaddstr(w, ++offsy + pos, xpos, smail.c_str());
			}
		}
	}
}


void screens::printfolders(bool widelist) {
	folders.read(gb.logfile);
	if ( gb.hidedevnull ) folders.remove("/dev/null");

	for ( Folders::const_iterator it=folders.begin();
		it!=folders.end(); ++it )
	{
		std::cout << std::setw(5) << (*it).vmail.size() << " " << (*it).name() << '\n';

		for ( std::vector<mail>::const_iterator p = (*it).vmail.begin();
			widelist &&
			p!=(*it).vmail.end(); ++p )
		{
			std::cout << "      " << (*p).from
			  << ": " << (*p).subj << '\n';
		}
	}
}

const std::string statustext(const gubbycfg& gb, const Folders& f) {
#ifndef USE_STRSTREAM
	std::stringstream status;
#else
	std::strstream status; // for GCC 2.95.2
#endif

	status <<
		gb.title <<
		" -- New mail: " <<
		f.totalmail() <<
		" in "<< f.size() <<
		" folder" <<
		( f.size()!=1 ? "s" : "" ) <<
		( gb.beep ? " (beep on)" : "" );
		
	return status.str();
}

void screens::startcurses(initcurs& curses) {
	curses.init();

	if ( curses.hascolors() ) {
		init_pair(1, COLOR_WHITE, COLOR_BLACK);
		init_pair(2, COLOR_WHITE, COLOR_BLUE);
		init_pair(3, COLOR_YELLOW, COLOR_RED);
	} else {
		init_pair(1, COLOR_WHITE, COLOR_BLACK);
		init_pair(2, COLOR_WHITE, COLOR_BLACK);
		init_pair(3, COLOR_WHITE, COLOR_BLACK);
	}
}

void screens::mainscreen(initcurs& curses) {
	screen scr(stdscr);
	intrflush(scr.ptr(), TRUE);

	unsigned int prevsize(0);
	bool forcedredraw(false);
	bool printscreen(true);
	bool shelledout(false);

	folders.read(gb.logfile);
	if ( gb.hidedevnull ) folders.remove("/dev/null");

	filechange logfile(gb.logfile);

	std::string topbar( statustext(gb, folders) );

	int x(0), y(0);
	getmaxyx(stdscr, y, x);
	std::string keyhelp( keysinfo() );

	do {
		if ( logfile.changed() ) {

			prevsize = folders.size();
			folders.read(gb.logfile);
			if ( gb.hidedevnull ) folders.remove("/dev/null");

			topbar = statustext(gb, folders);
			printscreen = true;

			if ( folders.size() == 0 ) {
				forcedredraw = true;
			}

			if ( gb.beep && folders.size() ) {

				if ( !(folders.newest()=="/dev/null" && gb.nobeepdevnull==true) )
					// beep on every new mail
					// ... except when folder is /dev/null and nobeepdevnull is set
					beep();
			}

			if ( curspos > folders.size() )
				curspos = folders.size();
		}

		if ( shelledout ) {
			wclear(scr.ptr());
			wrefresh(scr.ptr());
			shelledout = false;
			forcedredraw = true;

			// intrflush TRUE must be used if someone e.g.
			// does a ctrl+z (suspend) inside mutt, then
			// the wrong display is updated without this line
			intrflush(scr.ptr(), TRUE);
		}

		if ( scr.resized()
			|| forcedredraw
			/*|| prevsize > folders.size()*/ )
		{
			werase(scr.ptr());
			forcedredraw = false;
			printscreen = true;
			topbar = statustext(gb, folders);
			keyhelp = keysinfo();
		}

		if ( printscreen ) {
			scr.wprintintense(curses.hascolors(), 0, 0, topbar);
			
			if ( folders.size() ) {
				wprintfolders(scr.ptr(), gb.expanded, scr.width(), curses.hascolors());
				scr.wprintpointer(curses.hascolors(), curspos);
			}

			scr.wprintintense(curses.hascolors(), scr.height()-2, 0, keyhelp);
			scr.wprintversion(gb);

			wmove(scr.ptr(), (folders.size() ? 1+curspos : 0), scr.width()-1);
		}

		wrefresh(scr.ptr());

	} while ( !exitkey( keytoenum(getch()), forcedredraw, curses, printscreen, shelledout) );
}
